<?php

App::uses('ModelBehavior', 'Model');

class CustomFieldsDataBehavior extends ModelBehavior
{
    /**
     * Save the custom field data
     *
     * @param Model $model Model using this behavior
     * @param bool $created True if this save created a new record
     * @param array $options Options passed from Model::save().
     * @return bool
     * @see Model::save()
     */
    public function afterSave(Model $Model, $created, $options = array())
    {
        $return = parent::afterSave($Model, $created, $options);
        $result = true;

        if (! empty($Model->data[$Model->alias]['CustomFieldData'])) {
            // setup the model / model id
            $modelInfo = array(
                'model' => EvClassRegistry::getNameFromModel($Model),
                'model_id' => $Model->data[$Model->alias][$Model->primaryKey]
            );

            foreach ($Model->data[$Model->alias]['CustomFieldData'] as $field_id => $field) {

                $Model->data[$Model->alias]['CustomFieldData'][$field_id]['FieldData']['field_id'] = $field_id;
                $Model->data[$Model->alias]['CustomFieldData'][$field_id]['FieldData']['model'] = $modelInfo['model'];
                $Model->data[$Model->alias]['CustomFieldData'][$field_id]['FieldData']['model_id'] = $modelInfo['model_id'];
            }

            $FieldData = EvClassRegistry::init('EvCustomFields.FieldData');

            $result = $FieldData->saveMany(
                $Model->data[$Model->alias]['CustomFieldData']
            );
        }

        return ($return && $result);
    }

    /**
     * define an afterFind to retrieve hte custom field data for the item
     *
     * @param Model $model Model using this behavior
     * @param mixed $results The results of the find operation
     * @param bool $primary Whether this model is being queried directly (vs. being queried as an association)
     * @return mixed An array value will replace the value of $results - any other value will be ignored.
     */
    public function afterFind(Model $Model, $results, $primary = false)
    {
        // get all the model ids
        $modelIds = Hash::extract($results, '{n}.' . $Model->alias . '.' . $Model->primaryKey);

        if (empty($modelIds)) {
            return $results;
        }

        // get all the fields and data where id IN model_id
        $FieldData = EvClassRegistry::init('EvCustomFields.FieldData');
        $customData = $FieldData->find(
            'all',
            array(
                'conditions' => array(
                    'model' => EvClassRegistry::getNameFromModel($Model),
                    'model_id' => $modelIds
                )
            )
        );

        if (! empty($customData)) {

            $customData = Hash::combine(
                $customData,
                '{n}.FieldData.field_id',
                '{n}',
                '{n}.FieldData.model_id'
            );

            // loop the results and assign the field data to the result
            foreach ($results as $key => $item) {

                if (empty($item[$Model->alias][$Model->primaryKey])) {
                    continue;
                }

                if (! empty($customData[$item[$Model->alias][$Model->primaryKey]])) {

                    foreach ($customData[$item[$Model->alias][$Model->primaryKey]] as $field_id => $data) {

                        $results[$key]['CustomFieldData'][$field_id] = $data;
                    }
                }
            }
        }

        return $results;
    }
}
